--- old/usr/src/lib/libbe/common/be_create.c	2015-11-11 10:43:14.823895843 -0500
+++ new/usr/src/lib/libbe/common/be_create.c	2015-11-11 10:43:14.829905845 -0500
@@ -23,6 +23,7 @@
  * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
  * Copyright 2013 Nexenta Systems, Inc. All rights reserved.
  * Copyright (c) 2014 by Delphix. All rights reserved.
+ * Copyright 2015 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
  */
 
 /*
@@ -1618,7 +1619,6 @@
 	char		*zonepath_ds = NULL;
 	char		*mp = NULL;
 	zoneList_t	zlist = NULL;
-	zoneBrandList_t	*brands = NULL;
 	zfs_handle_t	*zhp = NULL;
 
 	/* If zones are not implemented, then get out. */
@@ -1626,20 +1626,12 @@
 		return (BE_SUCCESS);
 	}
 
-	/* Get list of supported brands */
-	if ((brands = be_get_supported_brandlist()) == NULL) {
-		be_print_err(gettext("be_destroy_zones: "
-		    "no supported brands\n"));
-		return (BE_SUCCESS);
-	}
-
 	/* Get handle to BE's root dataset */
 	if ((zhp = zfs_open(g_zfs, be_root_ds, ZFS_TYPE_FILESYSTEM)) ==
 	    NULL) {
 		be_print_err(gettext("be_destroy_zones: failed to "
 		    "open BE root dataset (%s): %s\n"), be_root_ds,
 		    libzfs_error_description(g_zfs));
-		z_free_brand_list(brands);
 		return (zfs_err_to_be_err(g_zfs));
 	}
 
@@ -1654,7 +1646,6 @@
 			    "mount the BE (%s) for zones processing.\n"),
 			    be_name);
 			ZFS_CLOSE(zhp);
-			z_free_brand_list(brands);
 			return (ret);
 		}
 	}
@@ -1663,11 +1654,9 @@
 	z_set_zone_root(mp);
 	free(mp);
 
-	/* Get list of supported zones. */
-	if ((zlist = z_get_nonglobal_zone_list_by_brand(brands)) == NULL) {
-		z_free_brand_list(brands);
+	/* Get list of zones. */
+	if ((zlist = z_get_nonglobal_branded_zone_list()) == NULL)
 		return (BE_SUCCESS);
-	}
 
 	/* Unmount the BE before destroying the zones in it. */
 	if (dd->force_unmount)
@@ -1680,6 +1669,17 @@
 
 	/* Iterate through the zones and destroy them. */
 	for (i = 0; (zonename = z_zlist_get_zonename(zlist, i)) != NULL; i++) {
+		boolean_t auto_create;
+
+		if (z_zlist_is_zone_auto_create_be(zlist, i, &auto_create) != 0) {
+			be_print_err(gettext("be_destroy_zones: failed to get "
+			    "auto-create-be brand property\n"));
+			ret = -1; // XXX
+			goto done;
+		}
+
+		if (!auto_create)
+			continue;
 
 		/* Skip zones that aren't at least installed */
 		if (z_zlist_get_current_state(zlist, i) < ZONE_STATE_INSTALLED)
@@ -1716,7 +1716,6 @@
 	}
 
 done:
-	z_free_brand_list(brands);
 	z_free_zone_list(zlist);
 
 	return (ret);
@@ -1908,7 +1907,6 @@
 	zfs_handle_t	*nbe_zhp = NULL;
 	zfs_handle_t	*z_zhp = NULL;
 	zoneList_t	zlist = NULL;
-	zoneBrandList_t	*brands = NULL;
 	boolean_t	mounted_here = B_FALSE;
 	char		*snap_name = NULL;
 
@@ -1917,13 +1915,6 @@
 		return (BE_SUCCESS);
 	}
 
-	/* Get list of supported brands */
-	if ((brands = be_get_supported_brandlist()) == NULL) {
-		be_print_err(gettext("be_copy_zones: "
-		    "no supported brands\n"));
-		return (BE_SUCCESS);
-	}
-
 	/* Get handle to origin BE's root dataset */
 	if ((obe_zhp = zfs_open(g_zfs, obe_root_ds, ZFS_TYPE_FILESYSTEM))
 	    == NULL) {
@@ -1971,8 +1962,8 @@
 
 	z_set_zone_root(temp_mntpt);
 
-	/* Get list of supported zones. */
-	if ((zlist = z_get_nonglobal_zone_list_by_brand(brands)) == NULL) {
+	/* Get list of zones. */
+	if ((zlist = z_get_nonglobal_branded_zone_list()) == NULL) {
 		ret = BE_SUCCESS;
 		goto done;
 	}
@@ -1982,6 +1973,16 @@
 		be_fs_list_data_t	fld = { 0 };
 		char			zonepath_ds[MAXPATHLEN];
 		char			*ds = NULL;
+		boolean_t		auto_create;
+
+		if (z_zlist_is_zone_auto_create_be(zlist, i, &auto_create) != 0) {
+			be_print_err(gettext("be_copy_zones: failed to get "
+			    "auto-create-be brand property\n"));
+			ret = -1; // XXX
+		}
+
+		if (!auto_create)
+			continue;
 
 		/* Get zonepath of zone */
 		zonepath = z_zlist_get_zonepath(zlist, i);
@@ -2249,8 +2250,6 @@
 
 done:
 	free(snap_name);
-	if (brands != NULL)
-		z_free_brand_list(brands);
 	if (zlist != NULL)
 		z_free_zone_list(zlist);
 
